@@ -160,6 +160,10 @@ class OptionTree:
                     Argument(Arguments.REMOVE_PROPERTY, str, Hints.REMOVE_PROPERTY, allow_repeats=True),
                 ], input_name=Arguments.PRINCIPAL),
                 Option(Subcommands.ACCESS, input_name=Arguments.PRINCIPAL),
+                Option(Subcommands.RESET, args=[
+                    Argument(Arguments.NEW_CLIENT_ID, str, Hints.Principals.Reset.CLIENT_ID),
+                    Argument(Arguments.NEW_CLIENT_SECRET, str, Hints.Principals.Reset.CLIENT_SECRET),
+                ], input_name=Arguments.PRINCIPAL),
             ]),
             Option(Commands.PRINCIPAL_ROLES, 'manage principal roles', children=[
                 Option(Subcommands.CREATE, args=[
@@ -273,5 +277,50 @@ class OptionTree:
                 Option(Subcommands.UPDATE, input_name=Arguments.PROFILE),
                 Option(Subcommands.GET, input_name=Arguments.PROFILE),
                 Option(Subcommands.LIST),
-            ])
+            ]),
+            Option(Commands.POLICIES, 'manage policies', children=[
+                Option(Subcommands.CREATE, args=[
+                    Argument(Arguments.CATALOG, str, Hints.CatalogRoles.CATALOG_NAME),
+                    Argument(Arguments.NAMESPACE, str, Hints.Grant.NAMESPACE),
+                    Argument(Arguments.POLICY_FILE, str, Hints.Policies.POLICY_FILE),
+                    Argument(Arguments.POLICY_TYPE, str, Hints.Policies.POLICY_TYPE),
+                    Argument(Arguments.POLICY_DESCRIPTION, str, Hints.Policies.POLICY_DESCRIPTION),
+                ], input_name=Arguments.POLICY),
+                Option(Subcommands.DELETE, args=[
+                    Argument(Arguments.CATALOG, str, Hints.CatalogRoles.CATALOG_NAME),
+                    Argument(Arguments.NAMESPACE, str, Hints.Grant.NAMESPACE),
+                    Argument(Arguments.DETACH_ALL, bool, Hints.Policies.DETACH_ALL),
+                ], input_name=Arguments.POLICY),
+                Option(Subcommands.GET, args=[
+                    Argument(Arguments.CATALOG, str, Hints.CatalogRoles.CATALOG_NAME),
+                    Argument(Arguments.NAMESPACE, str, Hints.Grant.NAMESPACE),
+                ], input_name=Arguments.POLICY),
+                Option(Subcommands.LIST, args=[
+                    Argument(Arguments.CATALOG, str, Hints.CatalogRoles.CATALOG_NAME),
+                    Argument(Arguments.NAMESPACE, str, Hints.Grant.NAMESPACE),
+                    Argument(Arguments.TARGET_NAME, str, Hints.Policies.TARGET_NAME),
+                    Argument(Arguments.APPLICABLE, bool, Hints.Policies.APPLICABLE),
+                    Argument(Arguments.POLICY_TYPE, str, Hints.Policies.POLICY_TYPE),
+                ]),
+                Option(Subcommands.UPDATE, args=[
+                    Argument(Arguments.CATALOG, str, Hints.CatalogRoles.CATALOG_NAME),
+                    Argument(Arguments.NAMESPACE, str, Hints.Grant.NAMESPACE),
+                    Argument(Arguments.POLICY_FILE, str, Hints.Policies.POLICY_FILE),
+                    Argument(Arguments.POLICY_DESCRIPTION, str, Hints.Policies.POLICY_DESCRIPTION),
+                ], input_name=Arguments.POLICY),
+                Option(Subcommands.ATTACH, args=[
+                    Argument(Arguments.CATALOG, str, Hints.CatalogRoles.CATALOG_NAME),
+                    Argument(Arguments.NAMESPACE, str, Hints.Grant.NAMESPACE),
+                    Argument(Arguments.ATTACHMENT_TYPE, str, Hints.Policies.ATTACHMENT_TYPE),
+                    Argument(Arguments.ATTACHMENT_PATH, str, Hints.Policies.ATTACHMENT_PATH),
+                    Argument(Arguments.PARAMETERS, str, Hints.Policies.PARAMETERS, allow_repeats=True),
+                ], input_name=Arguments.POLICY),
+                Option(Subcommands.DETACH, args=[
+                    Argument(Arguments.CATALOG, str, Hints.CatalogRoles.CATALOG_NAME),
+                    Argument(Arguments.NAMESPACE, str, Hints.Grant.NAMESPACE),
+                    Argument(Arguments.ATTACHMENT_TYPE, str, Hints.Policies.ATTACHMENT_TYPE),
+                    Argument(Arguments.ATTACHMENT_PATH, str, Hints.Policies.ATTACHMENT_PATH),
+                    Argument(Arguments.PARAMETERS, str, Hints.Policies.PARAMETERS, allow_repeats=True),
+                ], input_name=Arguments.POLICY),
+            ]),
         ]
