@@ -20,7 +20,7 @@ import argparse
 import sys
 from typing import List, Optional, Dict
 
-from cli.constants import Arguments
+from cli.constants import Arguments, DEFAULT_HEADER
 from cli.options.option_tree import OptionTree, Option, Argument
 
 
@@ -55,6 +55,18 @@ class Parser(object):
             str,
             hint="access token for token-based authentication",
         ),
+        Argument(
+            Arguments.REALM,
+            str,
+            hint="realm to use with header. if not defined will be used default realm from Polaris server. read more: https://polaris.apache.org/releases/1.1.0/configuration/",
+            default=None
+        ),
+        Argument(
+            Arguments.HEADER,
+            str,
+            hint="header is defining a header name to use as context header name. if not defined will be used default from Polaris server",
+            default=DEFAULT_HEADER
+        ),
         Argument(Arguments.PROFILE, str, hint="profile for token-based authentication"),
         Argument(Arguments.PROXY, str, hint="proxy URL"),
         Argument(Arguments.DEBUG, bool, hint="Enable debug mode"),
@@ -62,19 +74,6 @@ class Parser(object):
 
     @staticmethod
     def _build_parser() -> argparse.ArgumentParser:
-        parser = TreeHelpParser(description="Polaris CLI")
-
-        for arg in Parser._ROOT_ARGUMENTS:
-            kwargs = {"help": arg.hint}
-            if arg.default is not None:
-                kwargs["default"] = arg.default
-
-            if arg.type is bool:
-                kwargs["action"] = "store_true"
-            else:
-                kwargs["type"] = arg.type
-
-            parser.add_argument(arg.get_flag_name(), **kwargs)
 
         # Add everything from the option tree to the parser:
         def add_arguments(parser, args: List[Argument]):
@@ -114,6 +113,9 @@ class Parser(object):
                     )
                     recurse_options(children_subparser, option.children)
 
+        parser = TreeHelpParser(description="Polaris CLI")
+        add_arguments(parser, Parser._ROOT_ARGUMENTS)
+
         subparser = parser.add_subparsers(dest="command", required=False)
         recurse_options(subparser, OptionTree.get_tree())
         return parser
