@@ -141,8 +141,7 @@ class TestCliParsing(unittest.TestCase):
                 def _capture(*args, **kwargs):
                     client.call_tracker['_method'] = method_name
                     for i, arg in enumerate(args):
-                        if arg is not None:
-                            client.call_tracker[i] = arg
+                        client.call_tracker[i] = arg
 
                 return _capture
 
@@ -588,6 +587,37 @@ class TestCliParsing(unittest.TestCase):
                 (0, 'catalog.connection_config_info.uri'): 'u',
             })
 
+        check_arguments(
+            mock_execute(['principals', 'reset', 'test', '--new-client-id', 'e469c048cf866df1', '--new-client-secret', 'e469c048cf866dfae469c048cf866df1']),
+            'reset_credentials', {
+                (0, None): 'test',
+                (1, 'client_id'): 'e469c048cf866df1',
+                (1, 'client_secret'): 'e469c048cf866dfae469c048cf866df1',
+            })
+
+        check_arguments(
+            mock_execute(['principals', 'reset', 'test']),
+            'reset_credentials', {
+                (0, None): 'test',
+                (1, None): None,
+            })
+
+        check_arguments(
+            mock_execute(['principals', 'reset', 'test', '--new-client-id', 'e469c048cf866df1']),
+            'reset_credentials', {
+                (0, None): 'test',
+                (1, 'client_id'): 'e469c048cf866df1',
+                (1, 'client_secret'): None,
+            })
+
+        check_arguments(
+            mock_execute(['principals', 'reset', 'test', '--new-client-secret', 'e469c048cf866dfae469c048cf866df1']),
+            'reset_credentials', {
+                (0, None): 'test',
+                (1, 'client_id'): None,
+                (1, 'client_secret'): 'e469c048cf866dfae469c048cf866df1',
+            })
+
 
 if __name__ == '__main__':
     unittest.main()
\ No newline at end of file
