@@ -32,9 +32,11 @@ import org.apache.polaris.core.entity.PolarisBaseEntity;
 import org.apache.polaris.core.entity.PolarisEntitiesActiveKey;
 import org.apache.polaris.core.entity.PolarisEntityCore;
 import org.apache.polaris.core.entity.PolarisEntityId;
+import org.apache.polaris.core.entity.PolarisEntitySubType;
 import org.apache.polaris.core.entity.PolarisEntityType;
 import org.apache.polaris.core.entity.PolarisGrantRecord;
 import org.apache.polaris.core.entity.PolarisPrincipalSecrets;
+import org.apache.polaris.core.persistence.pagination.EntityIdToken;
 import org.apache.polaris.core.persistence.pagination.PageToken;
 import org.apache.polaris.core.policy.PolarisPolicyMappingRecord;
 import org.apache.polaris.core.policy.PolicyEntity;
@@ -288,13 +290,28 @@ public class PolarisEclipseLinkStore {
       long catalogId,
       long parentId,
       @Nonnull PolarisEntityType entityType,
+      @Nonnull PolarisEntitySubType entitySubType,
       @Nonnull PageToken pageToken) {
     diagnosticServices.check(session != null, "session_is_null");
     checkInitialized();
 
     // Currently check against ENTITIES not joining with ENTITIES_ACTIVE
     String hql =
-        "SELECT m from ModelEntity m where m.catalogId=:catalogId and m.parentId=:parentId and m.typeCode=:typeCode";
+        "SELECT m from ModelEntity m where"
+            + " m.catalogId=:catalogId and m.parentId=:parentId and m.typeCode=:typeCode";
+
+    if (entitySubType != PolarisEntitySubType.ANY_SUBTYPE) {
+      hql += " and m.subTypeCode=:subTypeCode";
+    }
+
+    var entityIdToken = pageToken.valueAs(EntityIdToken.class);
+    if (entityIdToken.isPresent()) {
+      hql += " and m.id > :tokenId";
+    }
+
+    if (pageToken.paginationRequested()) {
+      hql += " order by m.id asc";
+    }
 
     TypedQuery<ModelEntity> query =
         session
@@ -303,6 +320,15 @@ public class PolarisEclipseLinkStore {
             .setParameter("parentId", parentId)
             .setParameter("typeCode", entityType.getCode());
 
+    if (entitySubType != PolarisEntitySubType.ANY_SUBTYPE) {
+      query.setParameter("subTypeCode", entitySubType.getCode());
+    }
+
+    if (entityIdToken.isPresent()) {
+      long tokenId = entityIdToken.get().entityId();
+      query = query.setParameter("tokenId", tokenId);
+    }
+
     return query.getResultList();
   }
 
