@@ -37,6 +37,7 @@ import java.util.function.Function;
 import java.util.function.Predicate;
 import java.util.stream.Collectors;
 import org.apache.polaris.core.PolarisCallContext;
+import org.apache.polaris.core.PolarisDiagnostics;
 import org.apache.polaris.core.entity.EntityNameLookupRecord;
 import org.apache.polaris.core.entity.LocationBasedEntity;
 import org.apache.polaris.core.entity.PolarisBaseEntity;
@@ -46,6 +47,7 @@ import org.apache.polaris.core.entity.PolarisEntityCore;
 import org.apache.polaris.core.entity.PolarisEntityId;
 import org.apache.polaris.core.entity.PolarisEntitySubType;
 import org.apache.polaris.core.entity.PolarisEntityType;
+import org.apache.polaris.core.entity.PolarisEvent;
 import org.apache.polaris.core.entity.PolarisGrantRecord;
 import org.apache.polaris.core.entity.PolarisPrincipalSecrets;
 import org.apache.polaris.core.persistence.BaseMetaStoreManager;
@@ -65,7 +67,9 @@ import org.apache.polaris.core.storage.PolarisStorageConfigurationInfo;
 import org.apache.polaris.core.storage.PolarisStorageIntegration;
 import org.apache.polaris.core.storage.PolarisStorageIntegrationProvider;
 import org.apache.polaris.core.storage.StorageLocation;
+import org.apache.polaris.persistence.relational.jdbc.models.EntityNameLookupRecordConverter;
 import org.apache.polaris.persistence.relational.jdbc.models.ModelEntity;
+import org.apache.polaris.persistence.relational.jdbc.models.ModelEvent;
 import org.apache.polaris.persistence.relational.jdbc.models.ModelGrantRecord;
 import org.apache.polaris.persistence.relational.jdbc.models.ModelPolicyMappingRecord;
 import org.apache.polaris.persistence.relational.jdbc.models.ModelPrincipalAuthenticationData;
@@ -77,6 +81,7 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
 
   private static final Logger LOGGER = LoggerFactory.getLogger(JdbcBasePersistenceImpl.class);
 
+  private final PolarisDiagnostics diagnostics;
   private final DatasourceOperations datasourceOperations;
   private final PrincipalSecretsGenerator secretsGenerator;
   private final PolarisStorageIntegrationProvider storageIntegrationProvider;
@@ -87,11 +92,13 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
   private static final int MAX_LOCATION_COMPONENTS = 40;
 
   public JdbcBasePersistenceImpl(
+      PolarisDiagnostics diagnostics,
       DatasourceOperations databaseOperations,
       PrincipalSecretsGenerator secretsGenerator,
       PolarisStorageIntegrationProvider storageIntegrationProvider,
       String realmId,
       int schemaVersion) {
+    this.diagnostics = diagnostics;
     this.datasourceOperations = databaseOperations;
     this.secretsGenerator = secretsGenerator;
     this.storageIntegrationProvider = storageIntegrationProvider;
@@ -167,7 +174,7 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
       Connection connection,
       QueryAction queryAction)
       throws SQLException {
-    ModelEntity modelEntity = ModelEntity.fromEntity(entity);
+    ModelEntity modelEntity = ModelEntity.fromEntity(entity, schemaVersion);
     if (originalEntity == null) {
       try {
         List<Object> values =
@@ -175,7 +182,10 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
         queryAction.apply(
             connection,
             QueryGenerator.generateInsertQuery(
-                ModelEntity.ALL_COLUMNS, ModelEntity.TABLE_NAME, values, realmId));
+                ModelEntity.getAllColumnNames(schemaVersion),
+                ModelEntity.TABLE_NAME,
+                values,
+                realmId));
       } catch (SQLException e) {
         if (datasourceOperations.isConstraintViolation(e)) {
           PolarisBaseEntity existingEntity =
@@ -215,7 +225,10 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
             queryAction.apply(
                 connection,
                 QueryGenerator.generateUpdateQuery(
-                    ModelEntity.ALL_COLUMNS, ModelEntity.TABLE_NAME, values, params));
+                    ModelEntity.getAllColumnNames(schemaVersion),
+                    ModelEntity.TABLE_NAME,
+                    values,
+                    params));
         if (rowsUpdated == 0) {
           throw new RetryOnConcurrencyException(
               "Entity '%s' id '%s' concurrently modified; expected version %s",
@@ -244,9 +257,66 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
     }
   }
 
+  @Override
+  public void writeEvents(@Nonnull List<PolarisEvent> events) {
+    if (events.isEmpty()) {
+      return; // or throw if empty list is invalid
+    }
+
+    try {
+      // Generate the SQL using the first event as the reference
+      PreparedQuery firstPreparedQuery =
+          QueryGenerator.generateInsertQuery(
+              ModelEvent.ALL_COLUMNS,
+              ModelEvent.TABLE_NAME,
+              ModelEvent.fromEvent(events.getFirst())
+                  .toMap(datasourceOperations.getDatabaseType())
+                  .values()
+                  .stream()
+                  .toList(),
+              realmId);
+      String expectedSql = firstPreparedQuery.sql();
+
+      List<List<Object>> parametersList = new ArrayList<>();
+      parametersList.add(firstPreparedQuery.parameters());
+
+      // Process remaining events and verify SQL consistency
+      for (int i = 1; i < events.size(); i++) {
+        PolarisEvent event = events.get(i);
+        PreparedQuery pq =
+            QueryGenerator.generateInsertQuery(
+                ModelEvent.ALL_COLUMNS,
+                ModelEvent.TABLE_NAME,
+                ModelEvent.fromEvent(event)
+                    .toMap(datasourceOperations.getDatabaseType())
+                    .values()
+                    .stream()
+                    .toList(),
+                realmId);
+
+        if (!expectedSql.equals(pq.sql())) {
+          throw new RuntimeException("All events did not generate the same SQL");
+        }
+
+        parametersList.add(pq.parameters());
+      }
+
+      int totalUpdated =
+          datasourceOperations.executeBatchUpdate(
+              new QueryGenerator.PreparedBatchQuery(expectedSql, parametersList));
+
+      if (totalUpdated == 0) {
+        throw new SQLException("No events were inserted.");
+      }
+    } catch (SQLException e) {
+      throw new RuntimeException(
+          String.format("Failed to write events due to %s", e.getMessage()), e);
+    }
+  }
+
   @Override
   public void deleteEntity(@Nonnull PolarisCallContext callCtx, @Nonnull PolarisBaseEntity entity) {
-    ModelEntity modelEntity = ModelEntity.fromEntity(entity);
+    ModelEntity modelEntity = ModelEntity.fromEntity(entity, schemaVersion);
     Map<String, Object> params =
         Map.of(
             "id",
@@ -258,7 +328,7 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
     try {
       datasourceOperations.executeUpdate(
           QueryGenerator.generateDeleteQuery(
-              ModelEntity.ALL_COLUMNS, ModelEntity.TABLE_NAME, params));
+              ModelEntity.getAllColumnNames(schemaVersion), ModelEntity.TABLE_NAME, params));
     } catch (SQLException e) {
       throw new RuntimeException(
           String.format("Failed to delete entity due to %s", e.getMessage()), e);
@@ -306,7 +376,7 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
             datasourceOperations.execute(
                 connection,
                 QueryGenerator.generateDeleteQuery(
-                    ModelEntity.ALL_COLUMNS, ModelEntity.TABLE_NAME, params));
+                    ModelEntity.getAllColumnNames(schemaVersion), ModelEntity.TABLE_NAME, params));
             datasourceOperations.execute(
                 connection,
                 QueryGenerator.generateDeleteQuery(
@@ -338,7 +408,7 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
         Map.of("catalog_id", catalogId, "id", entityId, "type_code", typeCode, "realm_id", realmId);
     return getPolarisBaseEntity(
         QueryGenerator.generateSelectQuery(
-            ModelEntity.ALL_COLUMNS, ModelEntity.TABLE_NAME, params));
+            ModelEntity.getAllColumnNames(schemaVersion), ModelEntity.TABLE_NAME, params));
   }
 
   @Override
@@ -362,13 +432,13 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
             realmId);
     return getPolarisBaseEntity(
         QueryGenerator.generateSelectQuery(
-            ModelEntity.ALL_COLUMNS, ModelEntity.TABLE_NAME, params));
+            ModelEntity.getAllColumnNames(schemaVersion), ModelEntity.TABLE_NAME, params));
   }
 
   @Nullable
   private PolarisBaseEntity getPolarisBaseEntity(QueryGenerator.PreparedQuery query) {
     try {
-      var results = datasourceOperations.executeSelect(query, new ModelEntity());
+      var results = datasourceOperations.executeSelect(query, new ModelEntity(schemaVersion));
       if (results.isEmpty()) {
         return null;
       } else if (results.size() > 1) {
@@ -390,9 +460,10 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
   public List<PolarisBaseEntity> lookupEntities(
       @Nonnull PolarisCallContext callCtx, List<PolarisEntityId> entityIds) {
     if (entityIds == null || entityIds.isEmpty()) return new ArrayList<>();
-    PreparedQuery query = QueryGenerator.generateSelectQueryWithEntityIds(realmId, entityIds);
+    PreparedQuery query =
+        QueryGenerator.generateSelectQueryWithEntityIds(realmId, schemaVersion, entityIds);
     try {
-      return datasourceOperations.executeSelect(query, new ModelEntity());
+      return datasourceOperations.executeSelect(query, new ModelEntity(schemaVersion));
     } catch (SQLException e) {
       throw new RuntimeException(
           String.format("Failed to retrieve polaris entities due to %s", e.getMessage()), e);
@@ -408,7 +479,7 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
             .collect(
                 Collectors.toMap(
                     entry -> new PolarisEntityId(entry.getCatalogId(), entry.getId()),
-                    ModelEntity::fromEntity));
+                    entry -> ModelEntity.fromEntity(entry, schemaVersion)));
     return entityIds.stream()
         .map(
             entityId -> {
@@ -421,38 +492,13 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
         .collect(Collectors.toList());
   }
 
-  @Nonnull
-  @Override
-  public Page<EntityNameLookupRecord> listEntities(
-      @Nonnull PolarisCallContext callCtx,
+  private PreparedQuery buildEntityQuery(
       long catalogId,
       long parentId,
-      @Nonnull PolarisEntityType entityType,
-      @Nonnull PolarisEntitySubType entitySubType,
-      @Nonnull PageToken pageToken) {
-    // TODO: only fetch the properties required for creating an EntityNameLookupRecord
-    return loadEntities(
-        callCtx,
-        catalogId,
-        parentId,
-        entityType,
-        entitySubType,
-        entity -> true,
-        EntityNameLookupRecord::new,
-        pageToken);
-  }
-
-  @Nonnull
-  @Override
-  public <T> Page<T> loadEntities(
-      @Nonnull PolarisCallContext callCtx,
-      long catalogId,
-      long parentId,
-      @Nonnull PolarisEntityType entityType,
-      @Nonnull PolarisEntitySubType entitySubType,
-      @Nonnull Predicate<PolarisBaseEntity> entityFilter,
-      @Nonnull Function<PolarisBaseEntity, T> transformer,
-      @Nonnull PageToken pageToken) {
+      PolarisEntityType entityType,
+      PolarisEntitySubType entitySubType,
+      PageToken pageToken,
+      List<String> queryProjections) {
     Map<String, Object> whereEquals =
         Map.of(
             "catalog_id",
@@ -463,7 +509,6 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
             entityType.getCode(),
             "realm_id",
             realmId);
-    Map<String, Object> whereGreater;
 
     if (entitySubType != PolarisEntitySubType.ANY_SUBTYPE) {
       Map<String, Object> updatedWhereEquals = new HashMap<>(whereEquals);
@@ -471,9 +516,8 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
       whereEquals = updatedWhereEquals;
     }
 
-    // Limit can't be pushed down, due to client side filtering
-    // absence of transaction.
     String orderByColumnName = null;
+    Map<String, Object> whereGreater;
     if (pageToken.paginationRequested()) {
       orderByColumnName = ModelEntity.ID_COLUMN;
       whereGreater =
@@ -487,18 +531,67 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
       whereGreater = Map.of();
     }
 
+    return QueryGenerator.generateSelectQuery(
+        queryProjections, ModelEntity.TABLE_NAME, whereEquals, whereGreater, orderByColumnName);
+  }
+
+  @Nonnull
+  @Override
+  public Page<EntityNameLookupRecord> listEntities(
+      @Nonnull PolarisCallContext callCtx,
+      long catalogId,
+      long parentId,
+      @Nonnull PolarisEntityType entityType,
+      @Nonnull PolarisEntitySubType entitySubType,
+      @Nonnull PageToken pageToken) {
     try {
       PreparedQuery query =
-          QueryGenerator.generateSelectQuery(
-              ModelEntity.ALL_COLUMNS,
-              ModelEntity.TABLE_NAME,
-              whereEquals,
-              whereGreater,
-              orderByColumnName);
+          buildEntityQuery(
+              catalogId,
+              parentId,
+              entityType,
+              entitySubType,
+              pageToken,
+              ModelEntity.ENTITY_LOOKUP_COLUMNS);
+      AtomicReference<Page<EntityNameLookupRecord>> results = new AtomicReference<>();
+      datasourceOperations.executeSelectOverStream(
+          query,
+          new EntityNameLookupRecordConverter(),
+          stream -> {
+            results.set(
+                Page.mapped(pageToken, stream, Function.identity(), EntityIdToken::fromEntity));
+          });
+      return results.get();
+    } catch (SQLException e) {
+      throw new RuntimeException(
+          String.format("Failed to retrieve polaris entities due to %s", e.getMessage()), e);
+    }
+  }
+
+  @Nonnull
+  @Override
+  public <T> Page<T> loadEntities(
+      @Nonnull PolarisCallContext callCtx,
+      long catalogId,
+      long parentId,
+      @Nonnull PolarisEntityType entityType,
+      @Nonnull PolarisEntitySubType entitySubType,
+      @Nonnull Predicate<PolarisBaseEntity> entityFilter,
+      @Nonnull Function<PolarisBaseEntity, T> transformer,
+      @Nonnull PageToken pageToken) {
+    try {
+      PreparedQuery query =
+          buildEntityQuery(
+              catalogId,
+              parentId,
+              entityType,
+              entitySubType,
+              pageToken,
+              ModelEntity.getAllColumnNames(schemaVersion));
       AtomicReference<Page<T>> results = new AtomicReference<>();
       datasourceOperations.executeSelectOverStream(
           query,
-          new ModelEntity(),
+          new ModelEntity(schemaVersion),
           stream -> {
             var data = stream.filter(entityFilter);
             results.set(Page.mapped(pageToken, data, transformer, EntityIdToken::fromEntity));
@@ -519,7 +612,7 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
     PolarisBaseEntity b =
         getPolarisBaseEntity(
             QueryGenerator.generateSelectQuery(
-                ModelEntity.ALL_COLUMNS, ModelEntity.TABLE_NAME, params));
+                ModelEntity.getAllColumnNames(schemaVersion), ModelEntity.TABLE_NAME, params));
     return b == null ? 0 : b.getGrantRecordsVersion();
   }
 
@@ -633,8 +726,8 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
       var results =
           datasourceOperations.executeSelect(
               QueryGenerator.generateSelectQuery(
-                  ModelEntity.ALL_COLUMNS, ModelEntity.TABLE_NAME, params),
-              new ModelEntity());
+                  ModelEntity.getAllColumnNames(schemaVersion), ModelEntity.TABLE_NAME, params),
+              new ModelEntity(schemaVersion));
       return results != null && !results.isEmpty();
     } catch (SQLException e) {
       throw new RuntimeException(
@@ -644,7 +737,8 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
     }
   }
 
-  static int loadSchemaVersion(DatasourceOperations datasourceOperations) {
+  static int loadSchemaVersion(
+      DatasourceOperations datasourceOperations, boolean fallbackOnDoesNotExist) {
     PreparedQuery query = QueryGenerator.generateVersionQuery();
     try {
       List<SchemaVersion> schemaVersion =
@@ -654,11 +748,28 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
       }
       return schemaVersion.getFirst().getValue();
     } catch (SQLException e) {
+      if (fallbackOnDoesNotExist && datasourceOperations.isRelationDoesNotExist(e)) {
+        return SchemaVersion.MINIMUM.getValue();
+      }
       LOGGER.error("Failed to load schema version due to {}", e.getMessage(), e);
       throw new IllegalStateException("Failed to retrieve schema version", e);
     }
   }
 
+  static boolean entityTableExists(DatasourceOperations datasourceOperations) {
+    PreparedQuery query = QueryGenerator.generateEntityTableExistQuery();
+    try {
+      List<PolarisBaseEntity> entities =
+          datasourceOperations.executeSelect(query, new ModelEntity());
+      return entities != null && !entities.isEmpty();
+    } catch (SQLException e) {
+      if (datasourceOperations.isRelationDoesNotExist(e)) {
+        return false;
+      }
+      throw new IllegalStateException("Failed to check if Entities table exists", e);
+    }
+  }
+
   /** {@inheritDoc} */
   @Override
   public <T extends PolarisEntity & LocationBasedEntity>
@@ -674,9 +785,9 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
 
     PreparedQuery query =
         QueryGenerator.generateOverlapQuery(
-            realmId, entity.getCatalogId(), entity.getBaseLocation());
+            realmId, schemaVersion, entity.getCatalogId(), entity.getBaseLocation());
     try {
-      var results = datasourceOperations.executeSelect(query, new ModelEntity());
+      var results = datasourceOperations.executeSelect(query, new ModelEntity(schemaVersion));
       if (!results.isEmpty()) {
         StorageLocation entityLocation = StorageLocation.of(entity.getBaseLocation());
         for (PolarisBaseEntity result : results) {
@@ -773,6 +884,42 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
     return principalSecrets;
   }
 
+  @Nullable
+  @Override
+  public PolarisPrincipalSecrets storePrincipalSecrets(
+      @Nonnull PolarisCallContext callCtx,
+      long principalId,
+      @Nonnull String resolvedClientId,
+      String customClientSecret) {
+    PolarisPrincipalSecrets principalSecrets =
+        new PolarisPrincipalSecrets(principalId, resolvedClientId, customClientSecret);
+    try {
+      ModelPrincipalAuthenticationData modelPrincipalAuthenticationData =
+          ModelPrincipalAuthenticationData.fromPrincipalAuthenticationData(principalSecrets);
+      datasourceOperations.executeUpdate(
+          QueryGenerator.generateInsertQuery(
+              ModelPrincipalAuthenticationData.ALL_COLUMNS,
+              ModelPrincipalAuthenticationData.TABLE_NAME,
+              modelPrincipalAuthenticationData
+                  .toMap(datasourceOperations.getDatabaseType())
+                  .values()
+                  .stream()
+                  .toList(),
+              realmId));
+    } catch (SQLException e) {
+      LOGGER.error(
+          "Failed to reset PrincipalSecrets  for clientId: {}, due to {}",
+          resolvedClientId,
+          e.getMessage(),
+          e);
+      throw new RuntimeException(
+          String.format("Failed to reset PrincipalSecrets for clientId: %s", resolvedClientId), e);
+    }
+
+    // return those
+    return principalSecrets;
+  }
+
   @Nullable
   @Override
   public PolarisPrincipalSecrets rotatePrincipalSecrets(
@@ -785,24 +932,20 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
     PolarisPrincipalSecrets principalSecrets = loadPrincipalSecrets(callCtx, clientId);
 
     // should be found
-    callCtx
-        .getDiagServices()
-        .checkNotNull(
-            principalSecrets,
-            "cannot_find_secrets",
-            "client_id={} principalId={}",
-            clientId,
-            principalId);
+    diagnostics.checkNotNull(
+        principalSecrets,
+        "cannot_find_secrets",
+        "client_id={} principalId={}",
+        clientId,
+        principalId);
 
     // ensure principal id is matching
-    callCtx
-        .getDiagServices()
-        .check(
-            principalId == principalSecrets.getPrincipalId(),
-            "principal_id_mismatch",
-            "expectedId={} id={}",
-            principalId,
-            principalSecrets.getPrincipalId());
+    diagnostics.check(
+        principalId == principalSecrets.getPrincipalId(),
+        "principal_id_mismatch",
+        "expectedId={} id={}",
+        principalId,
+        principalSecrets.getPrincipalId());
 
     // rotate the secrets
     principalSecrets.rotateSecrets(oldSecretHash);
@@ -1119,7 +1262,7 @@ public class JdbcBasePersistenceImpl implements BasePersistence, IntegrationPers
       PolarisStorageIntegration<T> loadPolarisStorageIntegration(
           @Nonnull PolarisCallContext callContext, @Nonnull PolarisBaseEntity entity) {
     PolarisStorageConfigurationInfo storageConfig =
-        BaseMetaStoreManager.extractStorageConfiguration(callContext.getDiagServices(), entity);
+        BaseMetaStoreManager.extractStorageConfiguration(diagnostics, entity);
     return storageIntegrationProvider.getStorageIntegrationForConfig(storageConfig);
   }
 
