@@ -35,7 +35,25 @@ public class ModelEntity implements Converter<PolarisBaseEntity> {
 
   public static final String ID_COLUMN = "id";
 
-  public static final List<String> ALL_COLUMNS =
+  private static final List<String> ALL_COLUMNS =
+      List.of(
+          "id",
+          "catalog_id",
+          "parent_id",
+          "type_code",
+          "name",
+          "entity_version",
+          "sub_type_code",
+          "create_timestamp",
+          "drop_timestamp",
+          "purge_timestamp",
+          "to_purge_timestamp",
+          "last_update_timestamp",
+          "properties",
+          "internal_properties",
+          "grant_records_version");
+
+  private static final List<String> ALL_COLUMNS_V2 =
       List.of(
           "id",
           "catalog_id",
@@ -54,6 +72,17 @@ public class ModelEntity implements Converter<PolarisBaseEntity> {
           "grant_records_version",
           "location_without_scheme");
 
+  public static List<String> getAllColumnNames(int schemaVersion) {
+    if (schemaVersion < 2) {
+      return ALL_COLUMNS;
+    } else {
+      return ALL_COLUMNS_V2;
+    }
+  }
+
+  public static final List<String> ENTITY_LOOKUP_COLUMNS =
+      List.of("id", "catalog_id", "parent_id", "type_code", "name", "sub_type_code");
+
   // the id of the catalog associated to that entity. use 0 if this entity is top-level
   // like a catalog
   private long catalogId;
@@ -103,6 +132,16 @@ public class ModelEntity implements Converter<PolarisBaseEntity> {
   // location for the entity but without a scheme, when applicable
   private String locationWithoutScheme;
 
+  // schema version of the entity
+  // NOTE: this field is not stored in the database, but is used to handle schema changes
+  private int schemaVersion;
+
+  public ModelEntity(int schemaVersion) {
+    this.schemaVersion = schemaVersion;
+  }
+
+  public ModelEntity() {}
+
   public long getId() {
     return id;
   }
@@ -167,12 +206,17 @@ public class ModelEntity implements Converter<PolarisBaseEntity> {
     return locationWithoutScheme;
   }
 
+  public int getSchemaVersion() {
+    return schemaVersion;
+  }
+
   public static Builder builder() {
     return new Builder();
   }
 
   @Override
   public PolarisBaseEntity fromResultSet(ResultSet r) throws SQLException {
+
     var modelEntity =
         ModelEntity.builder()
             .catalogId(r.getObject("catalog_id", Long.class))
@@ -192,7 +236,8 @@ public class ModelEntity implements Converter<PolarisBaseEntity> {
             // JSONB: use getString(), not getObject().
             .internalProperties(r.getString("internal_properties"))
             .grantRecordsVersion(r.getObject("grant_records_version", Integer.class))
-            .locationWithoutScheme(r.getString("location_without_scheme"))
+            .locationWithoutScheme(
+                this.schemaVersion >= 2 ? r.getString("location_without_scheme") : null)
             .build();
 
     return toEntity(modelEntity);
@@ -221,7 +266,9 @@ public class ModelEntity implements Converter<PolarisBaseEntity> {
       map.put("internal_properties", this.getInternalProperties());
     }
     map.put("grant_records_version", this.getGrantRecordsVersion());
-    map.put("location_without_scheme", this.getLocationWithoutScheme());
+    if (this.getSchemaVersion() >= 2) {
+      map.put("location_without_scheme", this.getLocationWithoutScheme());
+    }
     return map;
   }
 
@@ -312,12 +359,17 @@ public class ModelEntity implements Converter<PolarisBaseEntity> {
       return this;
     }
 
+    public Builder schemaVersion(int schemaVersion) {
+      entity.schemaVersion = schemaVersion;
+      return this;
+    }
+
     public ModelEntity build() {
       return entity;
     }
   }
 
-  public static ModelEntity fromEntity(PolarisBaseEntity entity) {
+  public static ModelEntity fromEntity(PolarisBaseEntity entity, int schemaVersion) {
     var builder =
         ModelEntity.builder()
             .catalogId(entity.getCatalogId())
@@ -352,6 +404,8 @@ public class ModelEntity implements Converter<PolarisBaseEntity> {
               .withoutScheme());
     }
 
+    builder.schemaVersion(schemaVersion);
+
     return builder.build();
   }
 
