@@ -34,6 +34,7 @@ import org.apache.polaris.core.entity.PolarisEntityCore;
 import org.apache.polaris.core.entity.PolarisEntityId;
 import org.apache.polaris.core.entity.PolarisEntitySubType;
 import org.apache.polaris.core.entity.PolarisEntityType;
+import org.apache.polaris.core.entity.PolarisEvent;
 import org.apache.polaris.core.entity.PolarisGrantRecord;
 import org.apache.polaris.core.persistence.pagination.Page;
 import org.apache.polaris.core.persistence.pagination.PageToken;
@@ -136,6 +137,13 @@ public interface BasePersistence extends PolicyMappingPersistence {
   void writeToGrantRecords(
       @Nonnull PolarisCallContext callCtx, @Nonnull PolarisGrantRecord grantRec);
 
+  /**
+   * Write all events to the events table. This is an append-only operation.
+   *
+   * @param events events to persist
+   */
+  void writeEvents(@Nonnull List<PolarisEvent> events);
+
   /**
    * Delete this entity from the meta store.
    *
