@@ -24,20 +24,18 @@ import com.fasterxml.jackson.core.JsonProcessingException;
 import com.fasterxml.jackson.core.JsonToken;
 import com.fasterxml.jackson.core.type.TypeReference;
 import com.fasterxml.jackson.databind.DeserializationFeature;
-import com.fasterxml.jackson.databind.JsonMappingException;
 import com.fasterxml.jackson.databind.ObjectMapper;
 import jakarta.annotation.Nullable;
 import java.io.IOException;
 import java.util.Map;
 import org.apache.iceberg.rest.RESTSerializers;
-import org.apache.polaris.core.PolarisCallContext;
 import org.apache.polaris.core.entity.PolarisBaseEntity;
 import org.apache.polaris.core.entity.PolarisTaskConstants;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /** A mapper to serialize/deserialize polaris objects. */
-public class PolarisObjectMapperUtil {
+public final class PolarisObjectMapperUtil {
   private static final Logger LOGGER = LoggerFactory.getLogger(PolarisObjectMapperUtil.class);
 
   /** mapper, allows to serialize/deserialize properties to/from JSON */
@@ -50,42 +48,39 @@ public class PolarisObjectMapperUtil {
     return mapper;
   }
 
+  private PolarisObjectMapperUtil() {
+    // utility class
+  }
+
   /**
    * Given the internal property as a map of key/value pairs, serialize it to a String
    *
    * @param properties a map of key/value pairs
    * @return a String, the JSON representation of the map
    */
-  public static String serializeProperties(
-      PolarisCallContext callCtx, Map<String, String> properties) {
-
-    String jsonString = null;
+  public static String serializeProperties(Map<String, String> properties) {
     try {
       // Deserialize the JSON string to a Map<String, String>
-      jsonString = MAPPER.writeValueAsString(properties);
+      return MAPPER.writeValueAsString(properties);
     } catch (JsonProcessingException ex) {
-      callCtx.getDiagServices().fail("got_json_processing_exception", ex.getMessage());
+      throw new RuntimeException("serializeProperties failed: " + ex.getMessage(), ex);
     }
-
-    return jsonString;
   }
 
-  public static String serialize(PolarisCallContext callCtx, Object object) {
+  public static String serialize(Object object) {
     try {
       return MAPPER.writeValueAsString(object);
     } catch (JsonProcessingException e) {
-      callCtx.getDiagServices().fail("got_json_processing_exception", e.getMessage());
+      throw new RuntimeException("serialize failed: " + e.getMessage(), e);
     }
-    return "";
   }
 
-  public static <T> T deserialize(PolarisCallContext callCtx, String text, Class<T> klass) {
+  public static <T> T deserialize(String text, Class<T> klass) {
     try {
       return MAPPER.readValue(text, klass);
     } catch (JsonProcessingException e) {
-      callCtx.getDiagServices().fail("got_json_processing_exception", e.getMessage());
+      throw new RuntimeException("deserialize failed: " + e.getMessage(), e);
     }
-    return null;
   }
 
   /**
@@ -94,24 +89,13 @@ public class PolarisObjectMapperUtil {
    * @param properties a JSON string representing the set of properties
    * @return a Map of string
    */
-  public static Map<String, String> deserializeProperties(
-      PolarisCallContext callCtx, String properties) {
-
-    Map<String, String> retProperties = null;
+  public static Map<String, String> deserializeProperties(String properties) {
     try {
       // Deserialize the JSON string to a Map<String, String>
-      retProperties = MAPPER.readValue(properties, new TypeReference<>() {});
-    } catch (JsonMappingException ex) {
-      callCtx
-          .getDiagServices()
-          .fail("got_json_mapping_exception", "properties={}, ex={}", properties, ex);
+      return MAPPER.readValue(properties, new TypeReference<>() {});
     } catch (JsonProcessingException ex) {
-      callCtx
-          .getDiagServices()
-          .fail("got_json_processing_exception", "properties={}, ex={}", properties, ex);
+      throw new RuntimeException("deserializeProperties failed: " + ex.getMessage(), ex);
     }
-
-    return retProperties;
   }
 
   public static class TaskExecutionState {
@@ -187,8 +171,4 @@ public class PolarisObjectMapperUtil {
       return null;
     }
   }
-
-  long now() {
-    return 0;
-  }
 }
