@@ -28,6 +28,7 @@ import java.util.List;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.TimeUnit;
 import org.apache.polaris.core.PolarisCallContext;
+import org.apache.polaris.core.PolarisDiagnostics;
 import org.apache.polaris.core.config.BehaviorChangeConfiguration;
 import org.apache.polaris.core.config.FeatureConfiguration;
 import org.apache.polaris.core.config.RealmConfig;
@@ -41,16 +42,9 @@ import org.apache.polaris.core.persistence.dao.entity.ResolvedEntityResult;
 /** An in-memory entity cache with a limit of 100k entities and a 1h TTL. */
 public class InMemoryEntityCache implements EntityCache {
 
-  // cache mode
-  private EntityCacheMode cacheMode;
-
-  // the meta store manager
+  private final PolarisDiagnostics diagnostics;
   private final PolarisMetaStoreManager polarisMetaStoreManager;
-
-  // Caffeine cache to keep entries by id
   private final Cache<Long, ResolvedPolarisEntity> byId;
-
-  // index by name
   private final AbstractMap<EntityCacheByNameKey, ResolvedPolarisEntity> byName;
 
   /**
@@ -59,7 +53,10 @@ public class InMemoryEntityCache implements EntityCache {
    * @param polarisMetaStoreManager the meta store manager implementation
    */
   public InMemoryEntityCache(
-      @Nonnull RealmConfig realmConfig, @Nonnull PolarisMetaStoreManager polarisMetaStoreManager) {
+      @Nonnull PolarisDiagnostics diagnostics,
+      @Nonnull RealmConfig realmConfig,
+      @Nonnull PolarisMetaStoreManager polarisMetaStoreManager) {
+    this.diagnostics = diagnostics;
 
     // by name cache
     this.byName = new ConcurrentHashMap<>();
@@ -95,9 +92,6 @@ public class InMemoryEntityCache implements EntityCache {
 
     // remember the meta store manager
     this.polarisMetaStoreManager = polarisMetaStoreManager;
-
-    // enabled by default
-    this.cacheMode = EntityCacheMode.ENABLE;
   }
 
   /**
@@ -212,24 +206,6 @@ public class InMemoryEntityCache implements EntityCache {
         || entity.getTypeCode() != otherEntity.getTypeCode();
   }
 
-  /**
-   * Get the current cache mode
-   *
-   * @return the cache mode
-   */
-  public EntityCacheMode getCacheMode() {
-    return cacheMode;
-  }
-
-  /**
-   * Allows to change the caching mode for testing
-   *
-   * @param cacheMode the cache mode
-   */
-  public void setCacheMode(EntityCacheMode cacheMode) {
-    this.cacheMode = cacheMode;
-  }
-
   /**
    * Get a cache entity entry given the id of the entity
    *
@@ -344,16 +320,13 @@ public class InMemoryEntityCache implements EntityCache {
       }
 
       // assert that entity, grant records and version are all set
-      callContext.getDiagServices().checkNotNull(entity, "unexpected_null_entity");
-      callContext.getDiagServices().checkNotNull(grantRecords, "unexpected_null_grant_records");
-      callContext
-          .getDiagServices()
-          .check(grantRecordsVersion > 0, "unexpected_null_grant_records_version");
+      diagnostics.checkNotNull(entity, "unexpected_null_entity");
+      diagnostics.checkNotNull(grantRecords, "unexpected_null_grant_records");
+      diagnostics.check(grantRecordsVersion > 0, "unexpected_null_grant_records_version");
 
       // create new cache entry
       newCacheEntry =
-          new ResolvedPolarisEntity(
-              callContext.getDiagServices(), entity, grantRecords, grantRecordsVersion);
+          new ResolvedPolarisEntity(diagnostics, entity, grantRecords, grantRecordsVersion);
 
       // insert cache entry
       this.replaceCacheEntry(existingCacheEntry, newCacheEntry);
@@ -401,13 +374,12 @@ public class InMemoryEntityCache implements EntityCache {
       }
 
       // if found, setup entry
-      callContext.getDiagServices().checkNotNull(result.getEntity(), "entity_should_loaded");
-      callContext
-          .getDiagServices()
-          .checkNotNull(result.getEntityGrantRecords(), "entity_grant_records_should_loaded");
+      diagnostics.checkNotNull(result.getEntity(), "entity_should_loaded");
+      diagnostics.checkNotNull(
+          result.getEntityGrantRecords(), "entity_grant_records_should_loaded");
       entry =
           new ResolvedPolarisEntity(
-              callContext.getDiagServices(),
+              diagnostics,
               result.getEntity(),
               result.getEntityGrantRecords(),
               result.getGrantRecordsVersion());
@@ -458,15 +430,14 @@ public class InMemoryEntityCache implements EntityCache {
       }
 
       // validate return
-      callContext.getDiagServices().checkNotNull(result.getEntity(), "entity_should_loaded");
-      callContext
-          .getDiagServices()
-          .checkNotNull(result.getEntityGrantRecords(), "entity_grant_records_should_loaded");
+      diagnostics.checkNotNull(result.getEntity(), "entity_should_loaded");
+      diagnostics.checkNotNull(
+          result.getEntityGrantRecords(), "entity_grant_records_should_loaded");
 
       // if found, setup entry
       entry =
           new ResolvedPolarisEntity(
-              callContext.getDiagServices(),
+              diagnostics,
               result.getEntity(),
               result.getEntityGrantRecords(),
               result.getGrantRecordsVersion());
