@@ -22,13 +22,10 @@ package org.apache.polaris.core.persistence.resolver;
 import jakarta.annotation.Nonnull;
 import jakarta.annotation.Nullable;
 import jakarta.ws.rs.core.SecurityContext;
-import org.apache.polaris.core.context.CallContext;
 
 public interface ResolutionManifestFactory {
 
   @Nonnull
   PolarisResolutionManifest createResolutionManifest(
-      @Nonnull CallContext callContext,
-      @Nonnull SecurityContext securityContext,
-      @Nullable String referenceCatalogName);
+      @Nonnull SecurityContext securityContext, @Nullable String referenceCatalogName);
 }
