@@ -22,11 +22,8 @@ package org.apache.polaris.core.persistence.resolver;
 import jakarta.annotation.Nonnull;
 import jakarta.annotation.Nullable;
 import jakarta.ws.rs.core.SecurityContext;
-import org.apache.polaris.core.context.CallContext;
 
 public interface ResolverFactory {
   Resolver createResolver(
-      @Nonnull CallContext callContext,
-      @Nonnull SecurityContext securityContext,
-      @Nullable String referenceCatalogName);
+      @Nonnull SecurityContext securityContext, @Nullable String referenceCatalogName);
 }
