@@ -34,6 +34,7 @@ import org.apache.polaris.core.entity.PolarisEntityCore;
 import org.apache.polaris.core.entity.PolarisEntityId;
 import org.apache.polaris.core.entity.PolarisEntitySubType;
 import org.apache.polaris.core.entity.PolarisEntityType;
+import org.apache.polaris.core.entity.PolarisEvent;
 import org.apache.polaris.core.entity.PolarisGrantRecord;
 import org.apache.polaris.core.entity.PolarisPrincipalSecrets;
 import org.apache.polaris.core.persistence.EntityAlreadyExistsException;
@@ -273,6 +274,11 @@ public abstract class AbstractTransactionalPersistence implements TransactionalP
     runActionInTransaction(callCtx, () -> this.writeToGrantRecordsInCurrentTxn(callCtx, grantRec));
   }
 
+  @Override
+  public void writeEvents(@Nonnull List<PolarisEvent> events) {
+    throw new UnsupportedOperationException("Not implemented for transactional persistence.");
+  }
+
   /** {@inheritDoc} */
   @Override
   public void deleteEntity(@Nonnull PolarisCallContext callCtx, @Nonnull PolarisBaseEntity entity) {
