@@ -19,10 +19,10 @@
 package org.apache.polaris.core.storage;
 
 import jakarta.annotation.Nonnull;
-import java.util.EnumMap;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
-import org.apache.polaris.core.PolarisDiagnostics;
+import org.apache.polaris.core.config.RealmConfig;
 
 /**
  * Abstract of Polaris Storage Integration. It holds the reference to an object that having the
@@ -33,11 +33,17 @@ import org.apache.polaris.core.PolarisDiagnostics;
 public abstract class PolarisStorageIntegration<T extends PolarisStorageConfigurationInfo> {
 
   private final String integrationIdentifierOrId;
+  private final T config;
 
-  public PolarisStorageIntegration(String identifierOrId) {
+  public PolarisStorageIntegration(T config, String identifierOrId) {
+    this.config = config;
     this.integrationIdentifierOrId = identifierOrId;
   }
 
+  protected T config() {
+    return config;
+  }
+
   public String getStorageIdentifierOrId() {
     return integrationIdentifierOrId;
   }
@@ -45,20 +51,23 @@ public abstract class PolarisStorageIntegration<T extends PolarisStorageConfigur
   /**
    * Subscope the creds against the allowed read and write locations.
    *
-   * @param diagnostics the diagnostics service
-   * @param storageConfig storage configuration
+   * @param realmConfig the call context
    * @param allowListOperation whether to allow LIST on all the provided allowed read/write
    *     locations
    * @param allowedReadLocations a set of allowed to read locations
    * @param allowedWriteLocations a set of allowed to write locations
+   * @param refreshCredentialsEndpoint an optional endpoint to use for refreshing credentials. If
+   *     supported by the storage type it will be returned to the client in the appropriate
+   *     properties. The endpoint may be relative to the base URI and the client is responsible for
+   *     handling the relative path
    * @return An enum map including the scoped credentials
    */
-  public abstract EnumMap<StorageAccessProperty, String> getSubscopedCreds(
-      @Nonnull PolarisDiagnostics diagnostics,
-      @Nonnull T storageConfig,
+  public abstract AccessConfig getSubscopedCreds(
+      @Nonnull RealmConfig realmConfig,
       boolean allowListOperation,
       @Nonnull Set<String> allowedReadLocations,
-      @Nonnull Set<String> allowedWriteLocations);
+      @Nonnull Set<String> allowedWriteLocations,
+      Optional<String> refreshCredentialsEndpoint);
 
   /**
    * Validate access for the provided operation actions and locations.
@@ -94,12 +103,14 @@ public abstract class PolarisStorageIntegration<T extends PolarisStorageConfigur
   @Nonnull
   public abstract Map<String, Map<PolarisStorageActions, ValidationResult>>
       validateAccessToLocations(
+          @Nonnull RealmConfig realmConfig,
           @Nonnull T storageConfig,
           @Nonnull Set<PolarisStorageActions> actions,
           @Nonnull Set<String> locations);
 
   /**
-   * Result of calling {@link #validateAccessToLocations(PolarisStorageConfigurationInfo, Set, Set)}
+   * Result of calling {@link #validateAccessToLocations(RealmConfig,
+   * PolarisStorageConfigurationInfo, Set, Set)}
    */
   public static final class ValidationResult {
     private final boolean success;
