@@ -32,6 +32,7 @@ import com.azure.storage.file.datalake.DataLakeFileSystemClient;
 import com.azure.storage.file.datalake.DataLakeFileSystemClientBuilder;
 import com.azure.storage.file.datalake.models.DataLakeStorageException;
 import com.azure.storage.file.datalake.models.PathItem;
+import com.google.common.base.Strings;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.lang.annotation.ElementType;
@@ -42,26 +43,26 @@ import java.nio.charset.StandardCharsets;
 import java.time.Duration;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.EnumMap;
 import java.util.HashSet;
 import java.util.List;
-import java.util.Map;
+import java.util.Optional;
 import java.util.stream.Stream;
-import org.apache.polaris.core.PolarisDefaultDiagServiceImpl;
+import org.apache.polaris.core.storage.AccessConfig;
+import org.apache.polaris.core.storage.BaseStorageIntegrationTest;
 import org.apache.polaris.core.storage.StorageAccessProperty;
 import org.apache.polaris.core.storage.azure.AzureCredentialsStorageIntegration;
 import org.apache.polaris.core.storage.azure.AzureStorageConfigurationInfo;
 import org.assertj.core.api.Assertions;
 import org.assertj.core.api.Assumptions;
-import org.assertj.core.util.Strings;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.extension.ExtensionContext;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.Arguments;
 import org.junit.jupiter.params.provider.ArgumentsProvider;
 import org.junit.jupiter.params.provider.ArgumentsSource;
+import org.junit.jupiter.params.support.ParameterDeclarations;
 
-public class AzureCredentialStorageIntegrationTest {
+public class AzureCredentialStorageIntegrationTest extends BaseStorageIntegrationTest {
 
   private final String clientId = System.getenv("AZURE_CLIENT_ID");
   private final String clientSecret = System.getenv("AZURE_CLIENT_SECRET");
@@ -116,17 +117,17 @@ public class AzureCredentialStorageIntegrationTest {
 
     boolean isBlobService = service.equals("blob");
     List<String> allowedLoc =
-        Arrays.asList(
+        List.of(
             String.format(
                 "abfss://container@icebergdfsstorageacct.%s.core.windows.net/polaris-test/",
                 service));
-    Map<StorageAccessProperty, String> credsMap =
+    AccessConfig accessConfig =
         subscopedCredsForOperations(
             /* allowedReadLoc= */ allowedLoc,
             /* allowedWriteLoc= */ new ArrayList<>(),
             allowListAction);
-    Assertions.assertThat(credsMap).hasSize(2);
-    String sasToken = credsMap.get(StorageAccessProperty.AZURE_SAS_TOKEN);
+    Assertions.assertThat(accessConfig.credentials()).hasSize(2);
+    String sasToken = accessConfig.get(StorageAccessProperty.AZURE_SAS_TOKEN);
     Assertions.assertThat(sasToken).isNotNull();
     String serviceEndpoint =
         String.format("https://icebergdfsstorageacct.%s.core.windows.net", service);
@@ -187,11 +188,11 @@ public class AzureCredentialStorageIntegrationTest {
     String allowedPrefix = "polaris-test";
     String blockedPrefix = "blocked-prefix";
     List<String> allowedLoc =
-        Arrays.asList(
+        List.of(
             String.format(
                 "abfss://container@icebergdfsstorageacct.%s.core.windows.net/%s",
                 service, allowedPrefix));
-    Map<StorageAccessProperty, String> credsMap =
+    AccessConfig accessConfig =
         subscopedCredsForOperations(
             /* allowedReadLoc= */ allowedLoc,
             /* allowedWriteLoc= */ new ArrayList<>(),
@@ -199,7 +200,7 @@ public class AzureCredentialStorageIntegrationTest {
 
     BlobClient blobClient =
         createBlobClient(
-            credsMap.get(StorageAccessProperty.AZURE_SAS_TOKEN),
+            accessConfig.get(StorageAccessProperty.AZURE_SAS_TOKEN),
             "https://icebergdfsstorageacct.dfs.core.windows.net",
             "container",
             allowedPrefix);
@@ -230,7 +231,7 @@ public class AzureCredentialStorageIntegrationTest {
     // read fail because container is blocked
     BlobClient blobClientReadFail =
         createBlobClient(
-            credsMap.get(StorageAccessProperty.AZURE_SAS_TOKEN),
+            accessConfig.get(StorageAccessProperty.AZURE_SAS_TOKEN),
             String.format("https://icebergdfsstorageacct.%s.core.windows.net", service),
             "regtest",
             blockedPrefix);
@@ -257,11 +258,11 @@ public class AzureCredentialStorageIntegrationTest {
     String allowedPrefix = "polaris-test/scopedcreds/";
     String blockedPrefix = "blocked-prefix";
     List<String> allowedLoc =
-        Arrays.asList(
+        List.of(
             String.format(
                 "abfss://container@icebergdfsstorageacct.%s.core.windows.net/%s",
                 service, allowedPrefix));
-    Map<StorageAccessProperty, String> credsMap =
+    AccessConfig accessConfig =
         subscopedCredsForOperations(
             /* allowedReadLoc= */ new ArrayList<>(),
             /* allowedWriteLoc= */ allowedLoc,
@@ -270,13 +271,13 @@ public class AzureCredentialStorageIntegrationTest {
         String.format("https://icebergdfsstorageacct.%s.core.windows.net", service);
     BlobClient blobClient =
         createBlobClient(
-            credsMap.get(StorageAccessProperty.AZURE_SAS_TOKEN),
+            accessConfig.get(StorageAccessProperty.AZURE_SAS_TOKEN),
             serviceEndpoint,
             "container",
             allowedPrefix + "metadata/00000-65ffa17b-fe64-4c38-bcb9-06f9bd12aa2a.metadata.json");
     DataLakeFileClient fileClient =
         createDatalakeFileClient(
-            credsMap.get(StorageAccessProperty.AZURE_SAS_TOKEN),
+            accessConfig.get(StorageAccessProperty.AZURE_SAS_TOKEN),
             serviceEndpoint,
             "container",
             "polaris-test/scopedcreds/metadata",
@@ -311,13 +312,13 @@ public class AzureCredentialStorageIntegrationTest {
     String blockedContainer = "regtest";
     BlobClient blobClientWriteFail =
         createBlobClient(
-            credsMap.get(StorageAccessProperty.AZURE_SAS_TOKEN),
+            accessConfig.get(StorageAccessProperty.AZURE_SAS_TOKEN),
             serviceEndpoint,
             blockedContainer,
             blockedPrefix);
     DataLakeFileClient fileClientFail =
         createDatalakeFileClient(
-            credsMap.get(StorageAccessProperty.AZURE_SAS_TOKEN),
+            accessConfig.get(StorageAccessProperty.AZURE_SAS_TOKEN),
             serviceEndpoint,
             blockedContainer,
             "polaris-test/scopedcreds/metadata",
@@ -338,23 +339,22 @@ public class AzureCredentialStorageIntegrationTest {
     }
   }
 
-  private Map<StorageAccessProperty, String> subscopedCredsForOperations(
+  private AccessConfig subscopedCredsForOperations(
       List<String> allowedReadLoc, List<String> allowedWriteLoc, boolean allowListAction) {
-    List<String> allowedLoc = new ArrayList<>();
-    allowedLoc.addAll(allowedReadLoc);
-    allowedLoc.addAll(allowedWriteLoc);
     AzureStorageConfigurationInfo azureConfig =
-        new AzureStorageConfigurationInfo(allowedLoc, tenantId);
+        AzureStorageConfigurationInfo.builder()
+            .addAllAllowedLocations(allowedReadLoc)
+            .addAllAllowedLocations(allowedWriteLoc)
+            .tenantId(tenantId)
+            .build();
     AzureCredentialsStorageIntegration azureCredsIntegration =
-        new AzureCredentialsStorageIntegration();
-    EnumMap<StorageAccessProperty, String> credsMap =
-        azureCredsIntegration.getSubscopedCreds(
-            new PolarisDefaultDiagServiceImpl(),
-            azureConfig,
-            allowListAction,
-            new HashSet<>(allowedReadLoc),
-            new HashSet<>(allowedWriteLoc));
-    return credsMap;
+        new AzureCredentialsStorageIntegration(azureConfig);
+    return azureCredsIntegration.getSubscopedCreds(
+        EMPTY_REALM_CONFIG,
+        allowListAction,
+        new HashSet<>(allowedReadLoc),
+        new HashSet<>(allowedWriteLoc),
+        Optional.empty());
   }
 
   private BlobContainerClient createContainerClient(
@@ -404,7 +404,8 @@ public class AzureCredentialStorageIntegrationTest {
 
   protected static class AzureTestArgs implements ArgumentsProvider {
     @Override
-    public Stream<? extends Arguments> provideArguments(ExtensionContext extensionContext) {
+    public Stream<? extends Arguments> provideArguments(
+        ParameterDeclarations parameterDeclarations, ExtensionContext extensionContext) {
       return Stream.of(
           Arguments.of(/* allowedList= */ true, "blob"),
           Arguments.of(/* allowedList= */ false, "blob"),
