@@ -18,14 +18,15 @@
 # under the License.
 #
 
-if [ ! -d ~/polaris-venv ]; then
-  python3 -m venv ~/polaris-venv
+if [ ! -d ~/polaris/polaris-venv ]; then
+  python3 -m venv ~/polaris/polaris-venv
 fi
 
-. ~/polaris-venv/bin/activate
+. ~/polaris/polaris-venv/bin/activate
 
-pip install poetry==1.5.0
+pip install -r requirements.txt
 
-cd client/python
-python3 -m poetry install
-deactivate
\ No newline at end of file
+cd ../client/python
+python3 -m poetry install --no-root
+
+deactivate
