@@ -46,7 +46,7 @@ fi
 REGTEST_HOME=$(dirname $(realpath $0))
 cd ${REGTEST_HOME}
 
-export SPARK_VERSION=spark-3.5.5
+export SPARK_VERSION=spark-3.5.6
 export SPARK_DISTRIBUTION=${SPARK_VERSION}-bin-hadoop3
 export SPARK_LOCAL_HOSTNAME=localhost # avoid VPN messing up driver local IP address binding
 
@@ -59,7 +59,7 @@ fi
 if ! output=$(curl -X POST -H "Polaris-Realm: POLARIS" "http://${POLARIS_HOST:-localhost}:8181/api/catalog/v1/oauth/tokens" \
   -d "grant_type=client_credentials" \
   -d "client_id=root" \
-  -d "client_secret=secret" \
+  -d "client_secret=s3cr3t" \
   -d "scope=PRINCIPAL_ROLE:ALL"); then
   echo "Error: Failed to retrieve bearer token"
   exit 1
