@@ -30,7 +30,6 @@ import jakarta.enterprise.context.RequestScoped;
 import jakarta.enterprise.inject.Any;
 import jakarta.enterprise.inject.Instance;
 import jakarta.inject.Inject;
-import jakarta.ws.rs.WebApplicationException;
 import jakarta.ws.rs.core.HttpHeaders;
 import jakarta.ws.rs.core.Response;
 import jakarta.ws.rs.core.SecurityContext;
@@ -61,6 +60,7 @@ import org.apache.iceberg.rest.requests.UpdateTableRequest;
 import org.apache.iceberg.rest.responses.ConfigResponse;
 import org.apache.iceberg.rest.responses.ImmutableLoadCredentialsResponse;
 import org.apache.iceberg.rest.responses.LoadTableResponse;
+import org.apache.polaris.core.PolarisDiagnostics;
 import org.apache.polaris.core.auth.PolarisAuthorizer;
 import org.apache.polaris.core.auth.PolarisPrincipal;
 import org.apache.polaris.core.catalog.ExternalCatalogFactory;
@@ -84,6 +84,7 @@ import org.apache.polaris.service.catalog.api.IcebergRestConfigurationApiService
 import org.apache.polaris.service.catalog.common.CatalogAdapter;
 import org.apache.polaris.service.config.ReservedProperties;
 import org.apache.polaris.service.context.catalog.CallContextCatalogFactory;
+import org.apache.polaris.service.events.listeners.PolarisEventListener;
 import org.apache.polaris.service.http.IcebergHttpUtil;
 import org.apache.polaris.service.http.IfNoneMatch;
 import org.apache.polaris.service.types.CommitTableRequest;
@@ -138,6 +139,7 @@ public class IcebergCatalogAdapter
           .add(Endpoint.create("POST", ResourcePaths.V1_TRANSACTIONS_COMMIT))
           .build();
 
+  private final PolarisDiagnostics diagnostics;
   private final RealmContext realmContext;
   private final CallContext callContext;
   private final RealmConfig realmConfig;
@@ -151,9 +153,11 @@ public class IcebergCatalogAdapter
   private final ReservedProperties reservedProperties;
   private final CatalogHandlerUtils catalogHandlerUtils;
   private final Instance<ExternalCatalogFactory> externalCatalogFactories;
+  private final PolarisEventListener polarisEventListener;
 
   @Inject
   public IcebergCatalogAdapter(
+      PolarisDiagnostics diagnostics,
       RealmContext realmContext,
       CallContext callContext,
       CallContextCatalogFactory catalogFactory,
@@ -165,7 +169,9 @@ public class IcebergCatalogAdapter
       CatalogPrefixParser prefixParser,
       ReservedProperties reservedProperties,
       CatalogHandlerUtils catalogHandlerUtils,
-      @Any Instance<ExternalCatalogFactory> externalCatalogFactories) {
+      @Any Instance<ExternalCatalogFactory> externalCatalogFactories,
+      PolarisEventListener polarisEventListener) {
+    this.diagnostics = diagnostics;
     this.realmContext = realmContext;
     this.callContext = callContext;
     this.realmConfig = callContext.getRealmConfig();
@@ -179,6 +185,7 @@ public class IcebergCatalogAdapter
     this.reservedProperties = reservedProperties;
     this.catalogHandlerUtils = catalogHandlerUtils;
     this.externalCatalogFactories = externalCatalogFactories;
+    this.polarisEventListener = polarisEventListener;
   }
 
   /**
@@ -206,6 +213,7 @@ public class IcebergCatalogAdapter
     validatePrincipal(securityContext);
 
     return new IcebergCatalogHandler(
+        diagnostics,
         callContext,
         resolutionManifestFactory,
         metaStoreManager,
@@ -216,7 +224,8 @@ public class IcebergCatalogAdapter
         polarisAuthorizer,
         reservedProperties,
         catalogHandlerUtils,
-        externalCatalogFactories);
+        externalCatalogFactories,
+        polarisEventListener);
   }
 
   @Override
@@ -366,23 +375,14 @@ public class IcebergCatalogAdapter
                   prefix,
                   TableIdentifier.of(namespace, createTableRequest.name()));
           if (createTableRequest.stageCreate()) {
-            if (delegationModes.isEmpty()) {
-              return Response.ok(catalog.createTableStaged(ns, createTableRequest)).build();
-            } else {
-              return Response.ok(
-                      catalog.createTableStagedWithWriteDelegation(
-                          ns, createTableRequest, refreshCredentialsEndpoint))
-                  .build();
-            }
-          } else if (delegationModes.isEmpty()) {
-            LoadTableResponse response = catalog.createTableDirect(ns, createTableRequest);
-            return tryInsertETagHeader(
-                    Response.ok(response), response, namespace, createTableRequest.name())
+            return Response.ok(
+                    catalog.createTableStaged(
+                        ns, createTableRequest, delegationModes, refreshCredentialsEndpoint))
                 .build();
           } else {
             LoadTableResponse response =
-                catalog.createTableDirectWithWriteDelegation(
-                    ns, createTableRequest, refreshCredentialsEndpoint);
+                catalog.createTableDirect(
+                    ns, createTableRequest, delegationModes, refreshCredentialsEndpoint);
             return tryInsertETagHeader(
                     Response.ok(response), response, namespace, createTableRequest.name())
                 .build();
@@ -430,24 +430,20 @@ public class IcebergCatalogAdapter
         securityContext,
         prefix,
         catalog -> {
-          LoadTableResponse response;
+          Optional<LoadTableResponse> response =
+              catalog.loadTable(
+                  tableIdentifier,
+                  snapshots,
+                  ifNoneMatch,
+                  delegationModes,
+                  getRefreshCredentialsEndpoint(delegationModes, prefix, tableIdentifier));
 
-          if (delegationModes.isEmpty()) {
-            response =
-                catalog
-                    .loadTableIfStale(tableIdentifier, ifNoneMatch, snapshots)
-                    .orElseThrow(() -> new WebApplicationException(Response.Status.NOT_MODIFIED));
-          } else {
-            Optional<String> refreshCredentialsEndpoint =
-                getRefreshCredentialsEndpoint(delegationModes, prefix, tableIdentifier);
-            response =
-                catalog
-                    .loadTableWithAccessDelegationIfStale(
-                        tableIdentifier, ifNoneMatch, snapshots, refreshCredentialsEndpoint)
-                    .orElseThrow(() -> new WebApplicationException(Response.Status.NOT_MODIFIED));
+          if (response.isEmpty()) {
+            return Response.notModified().build();
           }
 
-          return tryInsertETagHeader(Response.ok(response), response, namespace, table).build();
+          return tryInsertETagHeader(Response.ok(response.get()), response.get(), namespace, table)
+              .build();
         });
   }
 
@@ -804,7 +800,7 @@ public class IcebergCatalogAdapter
     if (warehouse == null) {
       throw new BadRequestException("Please specify a warehouse");
     }
-    Resolver resolver = resolverFactory.createResolver(callContext, securityContext, warehouse);
+    Resolver resolver = resolverFactory.createResolver(securityContext, warehouse);
     ResolverStatus resolverStatus = resolver.resolveAll();
     if (!resolverStatus.getStatus().equals(ResolverStatus.StatusEnum.SUCCESS)) {
       throw new NotFoundException("Unable to find warehouse %s", warehouse);
