@@ -37,6 +37,8 @@ import org.apache.polaris.core.entity.PrincipalEntity;
 import org.apache.polaris.core.entity.PrincipalRoleEntity;
 import org.apache.polaris.core.persistence.dao.entity.PrivilegeResult;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.ValueSource;
 
 @QuarkusTest
 @TestProfile(PolarisAuthzTestBase.Profile.class)
@@ -49,10 +51,12 @@ public class PolarisAdminServiceAuthzTest extends PolarisAuthzTestBase {
     final PolarisPrincipal authenticatedPrincipal =
         PolarisPrincipal.of(principalEntity, activatedPrincipalRoles);
     return new PolarisAdminService(
+        diagServices,
         callContext,
         resolutionManifestFactory,
         metaStoreManager,
         userSecretsManager,
+        serviceIdentityProvider,
         securityContext(authenticatedPrincipal),
         polarisAuthorizer,
         reservedProperties);
@@ -131,7 +135,8 @@ public class PolarisAdminServiceAuthzTest extends PolarisAuthzTestBase {
         adminService.grantPrivilegeOnRootContainerToPrincipalRole(
             PRINCIPAL_ROLE2, PolarisPrivilege.CATALOG_DROP));
     final CatalogEntity newCatalog = new CatalogEntity.Builder().setName("new_catalog").build();
-    final CreateCatalogRequest createRequest = new CreateCatalogRequest(newCatalog.asCatalog());
+    final CreateCatalogRequest createRequest =
+        new CreateCatalogRequest(newCatalog.asCatalog(serviceIdentityProvider));
 
     doTestSufficientPrivileges(
         List.of(
@@ -150,7 +155,8 @@ public class PolarisAdminServiceAuthzTest extends PolarisAuthzTestBase {
   @Test
   public void testCreateCatalogInsufficientPrivileges() {
     final CatalogEntity newCatalog = new CatalogEntity.Builder().setName("new_catalog").build();
-    final CreateCatalogRequest createRequest = new CreateCatalogRequest(newCatalog.asCatalog());
+    final CreateCatalogRequest createRequest =
+        new CreateCatalogRequest(newCatalog.asCatalog(serviceIdentityProvider));
 
     doTestInsufficientPrivileges(
         List.of(
@@ -285,7 +291,8 @@ public class PolarisAdminServiceAuthzTest extends PolarisAuthzTestBase {
         adminService.grantPrivilegeOnRootContainerToPrincipalRole(
             PRINCIPAL_ROLE2, PolarisPrivilege.CATALOG_CREATE));
     final CatalogEntity newCatalog = new CatalogEntity.Builder().setName("new_catalog").build();
-    final CreateCatalogRequest createRequest = new CreateCatalogRequest(newCatalog.asCatalog());
+    final CreateCatalogRequest createRequest =
+        new CreateCatalogRequest(newCatalog.asCatalog(serviceIdentityProvider));
     adminService.createCatalog(createRequest);
 
     doTestSufficientPrivileges(
@@ -305,7 +312,8 @@ public class PolarisAdminServiceAuthzTest extends PolarisAuthzTestBase {
   @Test
   public void testDeleteCatalogInsufficientPrivileges() {
     final CatalogEntity newCatalog = new CatalogEntity.Builder().setName("new_catalog").build();
-    final CreateCatalogRequest createRequest = new CreateCatalogRequest(newCatalog.asCatalog());
+    final CreateCatalogRequest createRequest =
+        new CreateCatalogRequest(newCatalog.asCatalog(serviceIdentityProvider));
     adminService.createCatalog(createRequest);
 
     doTestInsufficientPrivileges(
@@ -1497,8 +1505,9 @@ public class PolarisAdminServiceAuthzTest extends PolarisAuthzTestBase {
             adminService.revokePrivilegeOnCatalogFromRole(CATALOG_NAME, CATALOG_ROLE1, privilege));
   }
 
-  @Test
-  public void testGrantPrivilegeOnNamespaceToRoleSufficientPrivileges() {
+  @ParameterizedTest(name = "{displayName}({0})")
+  @ValueSource(strings = {CATALOG_NAME, FEDERATED_CATALOG_NAME})
+  public void testGrantPrivilegeOnNamespaceToRoleSufficientPrivileges(String catalogName) {
     doTestSufficientPrivileges(
         List.of(
             PolarisPrivilege.CATALOG_MANAGE_ACCESS,
@@ -1506,16 +1515,39 @@ public class PolarisAdminServiceAuthzTest extends PolarisAuthzTestBase {
         () ->
             newTestAdminService(Set.of(PRINCIPAL_ROLE1))
                 .grantPrivilegeOnNamespaceToRole(
-                    CATALOG_NAME, CATALOG_ROLE2, NS1, PolarisPrivilege.CATALOG_MANAGE_ACCESS),
+                    catalogName, CATALOG_ROLE2, NS1, PolarisPrivilege.CATALOG_MANAGE_ACCESS),
         null, // cleanupAction
         (privilege) ->
-            adminService.grantPrivilegeOnCatalogToRole(CATALOG_NAME, CATALOG_ROLE1, privilege),
+            adminService.grantPrivilegeOnCatalogToRole(catalogName, CATALOG_ROLE1, privilege),
         (privilege) ->
-            adminService.revokePrivilegeOnCatalogFromRole(CATALOG_NAME, CATALOG_ROLE1, privilege));
+            adminService.revokePrivilegeOnCatalogFromRole(catalogName, CATALOG_ROLE1, privilege));
   }
 
   @Test
-  public void testGrantPrivilegeOnNamespaceToRoleInsufficientPrivileges() {
+  public void testGrantPrivilegeOnNamespaceToRoleSufficientPrivileges_FederationNestedNamespace() {
+    doTestSufficientPrivileges(
+        List.of(
+            PolarisPrivilege.CATALOG_MANAGE_ACCESS,
+            PolarisPrivilege.NAMESPACE_MANAGE_GRANTS_ON_SECURABLE),
+        () ->
+            newTestAdminService(Set.of(PRINCIPAL_ROLE1))
+                .grantPrivilegeOnNamespaceToRole(
+                    FEDERATED_CATALOG_NAME,
+                    CATALOG_ROLE2,
+                    NS1AA,
+                    PolarisPrivilege.CATALOG_MANAGE_ACCESS),
+        null, // cleanupAction
+        (privilege) ->
+            adminService.grantPrivilegeOnCatalogToRole(
+                FEDERATED_CATALOG_NAME, CATALOG_ROLE1, privilege),
+        (privilege) ->
+            adminService.revokePrivilegeOnCatalogFromRole(
+                FEDERATED_CATALOG_NAME, CATALOG_ROLE1, privilege));
+  }
+
+  @ParameterizedTest(name = "{displayName}({0})")
+  @ValueSource(strings = {CATALOG_NAME, FEDERATED_CATALOG_NAME})
+  public void testGrantPrivilegeOnNamespaceToRoleInsufficientPrivileges(String catalogName) {
     doTestInsufficientPrivileges(
         List.of(
             PolarisPrivilege.PRINCIPAL_MANAGE_GRANTS_FOR_GRANTEE,
@@ -1542,11 +1574,11 @@ public class PolarisAdminServiceAuthzTest extends PolarisAuthzTestBase {
         () ->
             newTestAdminService(Set.of(PRINCIPAL_ROLE1))
                 .grantPrivilegeOnNamespaceToRole(
-                    CATALOG_NAME, CATALOG_ROLE2, NS1, PolarisPrivilege.CATALOG_MANAGE_ACCESS),
+                    catalogName, CATALOG_ROLE2, NS1, PolarisPrivilege.CATALOG_MANAGE_ACCESS),
         (privilege) ->
-            adminService.grantPrivilegeOnCatalogToRole(CATALOG_NAME, CATALOG_ROLE1, privilege),
+            adminService.grantPrivilegeOnCatalogToRole(catalogName, CATALOG_ROLE1, privilege),
         (privilege) ->
-            adminService.revokePrivilegeOnCatalogFromRole(CATALOG_NAME, CATALOG_ROLE1, privilege));
+            adminService.revokePrivilegeOnCatalogFromRole(catalogName, CATALOG_ROLE1, privilege));
   }
 
   @Test
@@ -1605,8 +1637,9 @@ public class PolarisAdminServiceAuthzTest extends PolarisAuthzTestBase {
             adminService.revokePrivilegeOnCatalogFromRole(CATALOG_NAME, CATALOG_ROLE1, privilege));
   }
 
-  @Test
-  public void testGrantPrivilegeOnTableToRoleSufficientPrivileges() {
+  @ParameterizedTest(name = "{displayName}({0})")
+  @ValueSource(strings = {CATALOG_NAME, FEDERATED_CATALOG_NAME})
+  public void testGrantPrivilegeOnTableToRoleSufficientPrivileges(String catalogName) {
     doTestSufficientPrivileges(
         List.of(
             PolarisPrivilege.CATALOG_MANAGE_ACCESS,
@@ -1614,15 +1647,15 @@ public class PolarisAdminServiceAuthzTest extends PolarisAuthzTestBase {
         () ->
             newTestAdminService(Set.of(PRINCIPAL_ROLE1))
                 .grantPrivilegeOnTableToRole(
-                    CATALOG_NAME,
+                    catalogName,
                     CATALOG_ROLE2,
                     TABLE_NS1_1,
                     PolarisPrivilege.CATALOG_MANAGE_ACCESS),
         null, // cleanupAction
         (privilege) ->
-            adminService.grantPrivilegeOnCatalogToRole(CATALOG_NAME, CATALOG_ROLE1, privilege),
+            adminService.grantPrivilegeOnCatalogToRole(catalogName, CATALOG_ROLE1, privilege),
         (privilege) ->
-            adminService.revokePrivilegeOnCatalogFromRole(CATALOG_NAME, CATALOG_ROLE1, privilege));
+            adminService.revokePrivilegeOnCatalogFromRole(catalogName, CATALOG_ROLE1, privilege));
   }
 
   @Test
