@@ -19,6 +19,7 @@
 package org.apache.polaris.service.admin;
 
 import static org.apache.iceberg.types.Types.NestedField.required;
+import static org.apache.polaris.core.entity.CatalogEntity.DEFAULT_BASE_LOCATION_KEY;
 
 import com.google.auth.oauth2.AccessToken;
 import com.google.auth.oauth2.GoogleCredentials;
@@ -28,6 +29,7 @@ import jakarta.annotation.Nonnull;
 import jakarta.enterprise.context.RequestScoped;
 import jakarta.enterprise.inject.Alternative;
 import jakarta.inject.Inject;
+import jakarta.ws.rs.container.ContainerRequestContext;
 import jakarta.ws.rs.core.SecurityContext;
 import java.io.IOException;
 import java.util.Date;
@@ -46,8 +48,13 @@ import org.apache.iceberg.exceptions.ForbiddenException;
 import org.apache.iceberg.types.Types;
 import org.apache.polaris.core.PolarisCallContext;
 import org.apache.polaris.core.PolarisDiagnostics;
+import org.apache.polaris.core.admin.model.AuthenticationParameters;
+import org.apache.polaris.core.admin.model.ConnectionConfigInfo;
 import org.apache.polaris.core.admin.model.CreateCatalogRequest;
+import org.apache.polaris.core.admin.model.ExternalCatalog;
 import org.apache.polaris.core.admin.model.FileStorageConfigInfo;
+import org.apache.polaris.core.admin.model.IcebergRestConnectionConfigInfo;
+import org.apache.polaris.core.admin.model.OAuthClientCredentialsParameters;
 import org.apache.polaris.core.admin.model.PrincipalWithCredentials;
 import org.apache.polaris.core.admin.model.PrincipalWithCredentialsCredentials;
 import org.apache.polaris.core.admin.model.StorageConfigInfo;
@@ -65,6 +72,7 @@ import org.apache.polaris.core.entity.PolarisEntityType;
 import org.apache.polaris.core.entity.PolarisPrivilege;
 import org.apache.polaris.core.entity.PrincipalEntity;
 import org.apache.polaris.core.entity.PrincipalRoleEntity;
+import org.apache.polaris.core.identity.provider.ServiceIdentityProvider;
 import org.apache.polaris.core.persistence.MetaStoreManagerFactory;
 import org.apache.polaris.core.persistence.PolarisMetaStoreManager;
 import org.apache.polaris.core.persistence.dao.entity.BaseResult;
@@ -87,7 +95,7 @@ import org.apache.polaris.service.catalog.policy.PolicyCatalog;
 import org.apache.polaris.service.config.ReservedProperties;
 import org.apache.polaris.service.context.catalog.CallContextCatalogFactory;
 import org.apache.polaris.service.context.catalog.PolarisCallContextCatalogFactory;
-import org.apache.polaris.service.events.PolarisEventListener;
+import org.apache.polaris.service.events.listeners.PolarisEventListener;
 import org.apache.polaris.service.storage.PolarisStorageIntegrationProviderImpl;
 import org.apache.polaris.service.task.TaskExecutor;
 import org.apache.polaris.service.types.PolicyIdentifier;
@@ -119,11 +127,14 @@ public abstract class PolarisAuthzTestBase {
               "polaris.features.\"ENFORCE_PRINCIPAL_CREDENTIAL_ROTATION_REQUIRED_CHECKING\"",
               "true")
           .put("polaris.features.\"DROP_WITH_PURGE_ENABLED\"", "true")
+          .put("polaris.behavior-changes.\"ALLOW_NAMESPACE_CUSTOM_LOCATION\"", "true")
+          .put("polaris.features.\"ENABLE_CATALOG_FEDERATION\"", "true")
           .build();
     }
   }
 
   protected static final String CATALOG_NAME = "polaris-catalog";
+  protected static final String FEDERATED_CATALOG_NAME = "federated-polaris-catalog";
   protected static final String PRINCIPAL_NAME = "snowman";
 
   // catalog_role1 will be assigned only to principal_role1 and
@@ -185,6 +196,7 @@ public abstract class PolarisAuthzTestBase {
   @Inject protected ResolutionManifestFactory resolutionManifestFactory;
   @Inject protected CallContextCatalogFactory callContextCatalogFactory;
   @Inject protected UserSecretsManagerFactory userSecretsManagerFactory;
+  @Inject protected ServiceIdentityProvider serviceIdentityProvider;
   @Inject protected PolarisDiagnostics diagServices;
   @Inject protected FileIOFactory fileIOFactory;
   @Inject protected PolarisEventListener polarisEventListener;
@@ -200,6 +212,7 @@ public abstract class PolarisAuthzTestBase {
   protected PolarisMetaStoreManager metaStoreManager;
   protected UserSecretsManager userSecretsManager;
   protected PolarisBaseEntity catalogEntity;
+  protected PolarisBaseEntity federatedCatalogEntity;
   protected PrincipalEntity principalEntity;
   protected CallContext callContext;
   protected RealmConfig realmConfig;
@@ -225,15 +238,16 @@ public abstract class PolarisAuthzTestBase {
 
     RealmContext realmContext = testInfo::getDisplayName;
     QuarkusMock.installMockForType(realmContext, RealmContext.class);
+    ContainerRequestContext containerRequestContext = Mockito.mock(ContainerRequestContext.class);
+    Mockito.when(containerRequestContext.getProperty(Mockito.anyString()))
+        .thenReturn("request-id-1");
+    QuarkusMock.installMockForType(containerRequestContext, ContainerRequestContext.class);
     metaStoreManager = managerFactory.getOrCreateMetaStoreManager(realmContext);
     userSecretsManager = userSecretsManagerFactory.getOrCreateUserSecretsManager(realmContext);
 
     polarisContext =
         new PolarisCallContext(
-            realmContext,
-            managerFactory.getOrCreateSession(realmContext),
-            diagServices,
-            configurationStore);
+            realmContext, managerFactory.getOrCreateSession(realmContext), configurationStore);
 
     callContext = polarisContext;
     realmConfig = polarisContext.getRealmConfig();
@@ -246,20 +260,69 @@ public abstract class PolarisAuthzTestBase {
 
     this.adminService =
         new PolarisAdminService(
+            diagServices,
             callContext,
             resolutionManifestFactory,
             metaStoreManager,
             userSecretsManager,
+            serviceIdentityProvider,
             securityContext(authenticatedRoot),
             polarisAuthorizer,
             reservedProperties);
 
     String storageLocation = "file:///tmp/authz";
+    String storageLocationForFederatedCatalog = "file:///tmp/authz_federated";
     FileStorageConfigInfo storageConfigModel =
         FileStorageConfigInfo.builder()
             .setStorageType(StorageConfigInfo.StorageTypeEnum.FILE)
             .setAllowedLocations(List.of(storageLocation, "file:///tmp/authz"))
             .build();
+    FileStorageConfigInfo storageConfigModelForFederatedCatalog =
+        FileStorageConfigInfo.builder()
+            .setStorageType(StorageConfigInfo.StorageTypeEnum.FILE)
+            .setAllowedLocations(
+                List.of(storageLocationForFederatedCatalog, "file:///tmp/authz_federated"))
+            .build();
+    ConnectionConfigInfo connectionConfigInfo =
+        IcebergRestConnectionConfigInfo.builder(
+                ConnectionConfigInfo.ConnectionTypeEnum.ICEBERG_REST)
+            .setUri("https://myorg-my_account.snowflakecomputing.com/polaris/api/catalog")
+            .setRemoteCatalogName("my-remote-catalog")
+            .setAuthenticationParameters(
+                OAuthClientCredentialsParameters.builder(
+                        AuthenticationParameters.AuthenticationTypeEnum.OAUTH)
+                    .setClientId("my-client-id")
+                    .setClientSecret("my-client-secret")
+                    .setScopes(List.of("PRINCIPAL_ROLE:ALL"))
+                    .build())
+            .build();
+    CatalogEntity externalCatalogEntity =
+        new CatalogEntity.Builder()
+            .setName(FEDERATED_CATALOG_NAME)
+            .setCatalogType("EXTERNAL")
+            .setDefaultBaseLocation(storageLocationForFederatedCatalog)
+            .setStorageConfigurationInfo(
+                realmConfig,
+                storageConfigModelForFederatedCatalog,
+                storageLocationForFederatedCatalog)
+            .addProperty("polaris.config.enable-sub-catalog-rbac-for-federated-catalogs", "true")
+            .build();
+    ExternalCatalog externalCatalog =
+        ExternalCatalog.builder()
+            .setName(externalCatalogEntity.getName())
+            .setType(ExternalCatalog.TypeEnum.EXTERNAL)
+            .setProperties(
+                org.apache.polaris.core.admin.model.CatalogProperties.builder(
+                        externalCatalogEntity.getPropertiesAsMap().get(DEFAULT_BASE_LOCATION_KEY))
+                    .putAll(externalCatalogEntity.getPropertiesAsMap())
+                    .build())
+            .setCreateTimestamp(externalCatalogEntity.getCreateTimestamp())
+            .setLastUpdateTimestamp(externalCatalogEntity.getLastUpdateTimestamp())
+            .setEntityVersion(externalCatalogEntity.getEntityVersion())
+            .setStorageConfigInfo(storageConfigModelForFederatedCatalog)
+            .setConnectionConfigInfo(connectionConfigInfo)
+            .build();
+
     catalogEntity =
         adminService.createCatalog(
             new CreateCatalogRequest(
@@ -269,7 +332,8 @@ public abstract class PolarisAuthzTestBase {
                     .setDefaultBaseLocation(storageLocation)
                     .setStorageConfigurationInfo(realmConfig, storageConfigModel, storageLocation)
                     .build()
-                    .asCatalog()));
+                    .asCatalog(serviceIdentityProvider)));
+    federatedCatalogEntity = adminService.createCatalog(new CreateCatalogRequest(externalCatalog));
 
     initBaseCatalog();
 
@@ -292,6 +356,10 @@ public abstract class PolarisAuthzTestBase {
         CATALOG_NAME, new CatalogRoleEntity.Builder().setName(CATALOG_ROLE2).build());
     adminService.createCatalogRole(
         CATALOG_NAME, new CatalogRoleEntity.Builder().setName(CATALOG_ROLE_SHARED).build());
+    adminService.createCatalogRole(
+        FEDERATED_CATALOG_NAME, new CatalogRoleEntity.Builder().setName(CATALOG_ROLE1).build());
+    adminService.createCatalogRole(
+        FEDERATED_CATALOG_NAME, new CatalogRoleEntity.Builder().setName(CATALOG_ROLE2).build());
 
     assertSuccess(adminService.assignPrincipalRole(PRINCIPAL_NAME, PRINCIPAL_ROLE1));
     assertSuccess(adminService.assignPrincipalRole(PRINCIPAL_NAME, PRINCIPAL_ROLE2));
@@ -308,6 +376,12 @@ public abstract class PolarisAuthzTestBase {
     assertSuccess(
         adminService.assignCatalogRoleToPrincipalRole(
             PRINCIPAL_ROLE2, CATALOG_NAME, CATALOG_ROLE_SHARED));
+    assertSuccess(
+        adminService.assignCatalogRoleToPrincipalRole(
+            PRINCIPAL_ROLE1, FEDERATED_CATALOG_NAME, CATALOG_ROLE1));
+    assertSuccess(
+        adminService.assignCatalogRoleToPrincipalRole(
+            PRINCIPAL_ROLE2, FEDERATED_CATALOG_NAME, CATALOG_ROLE2));
 
     // Do some shared setup with non-authz-aware baseCatalog.
     baseCatalog.createNamespace(NS1);
@@ -395,9 +469,8 @@ public abstract class PolarisAuthzTestBase {
   protected @Nonnull Set<String> loadPrincipalRolesNames(PolarisPrincipal p) {
     PolarisBaseEntity principal =
         metaStoreManager
-            .loadEntity(
-                callContext.getPolarisCallContext(), 0L, p.getId(), PolarisEntityType.PRINCIPAL)
-            .getEntity();
+            .findPrincipalByName(callContext.getPolarisCallContext(), p.getName())
+            .orElseThrow();
     return metaStoreManager
         .loadGrantsToGrantee(callContext.getPolarisCallContext(), principal)
         .getGrantRecords()
@@ -450,9 +523,10 @@ public abstract class PolarisAuthzTestBase {
     Mockito.when(securityContext.isUserInRole(Mockito.anyString())).thenReturn(true);
     PolarisPassthroughResolutionView passthroughView =
         new PolarisPassthroughResolutionView(
-            callContext, resolutionManifestFactory, securityContext, CATALOG_NAME);
+            resolutionManifestFactory, securityContext, CATALOG_NAME);
     this.baseCatalog =
         new IcebergCatalog(
+            diagServices,
             storageCredentialCache,
             resolverFactory,
             metaStoreManager,
@@ -479,11 +553,12 @@ public abstract class PolarisAuthzTestBase {
 
     @SuppressWarnings("unused") // Required by CDI
     protected TestPolarisCallContextCatalogFactory() {
-      this(null, null, null, null, null, null);
+      this(null, null, null, null, null, null, null);
     }
 
     @Inject
     public TestPolarisCallContextCatalogFactory(
+        PolarisDiagnostics diagnostics,
         StorageCredentialCache storageCredentialCache,
         ResolverFactory resolverFactory,
         MetaStoreManagerFactory metaStoreManagerFactory,
@@ -491,6 +566,7 @@ public abstract class PolarisAuthzTestBase {
         FileIOFactory fileIOFactory,
         PolarisEventListener polarisEventListener) {
       super(
+          diagnostics,
           storageCredentialCache,
           resolverFactory,
           metaStoreManagerFactory,
